@@ -1,10 +1,8 @@
 #!/usr/bin/env perl
 
-use feature qw/say unicode_strings/;
-use open qw(:std :utf8);
 use strict;
 use warnings;
-use warnings qw(FATAL utf8);
+use warnings  qw(FATAL utf8);    # Fatalize encoding glitches.
 
 use Getopt::Long;
 
@@ -21,6 +19,7 @@ my(%option);
 if ($option_parser -> getoptions
 (
 	\%option,
+	'authortest=i',
 	'help',
 ) )
 {
@@ -46,6 +45,7 @@ generate.demo.pl - Generate GraphViz2::Marpa's html/index.html.
 generate.demo.pl [options]
 
 	Options:
+	-authortest $Boolean
 	-help
 
 Exit value: 0 for success, 1 for failure. Die upon error.
@@ -54,6 +54,14 @@ Exit value: 0 for success, 1 for failure. Die upon error.
 
 =over 4
 
+=item o -authortest $Boolean
+
+Specify whether (1) or not (0) to generate the author tests demo age.
+
+See scripts/generate.demo.sh.
+
+Default: 0.
+
 =item o -help
 
 Print help and exit.
